@*****************************************************************************@

@------------  Use of Fund Resources _ M. Goretti  ------------@

@********************Program with no autoregressive process*******************@


@ Gibbs sampling estimation of the time varying transition probability (TVTP) @
@ model in Filardo, A.J. and Gordon, S.F. (1998) "Business Cycle Durations",  @
@ Journal of Econometrics, 85, 99-123                                         @
@ and Martin Ellison, EUI, April 2000                       @

@      Inputs                                                                 @

@      cred            Fund credit EMs, t x 1 vector                          @
@      cabal           Changes in CA EMs, t x 1 vector                        @
@      capfl           Capital flows EMs, t x 1 vector                        @
@      corpchasp       Corporate-charges spreads EMs, t x 1 vector            @
@      reer            reer EMs, t x 1 vector                                 @
@      termst          term structure EMs, t x 1 vector                       @
@      textdebt        total ext. debt EMs, t x 1 vector                      @
@      STextdebt       ST ext. debt EMs, t x 1 vector                         @
@      reserv          reserves EMs, t x 1 vector                             @
@      emp             EMP index EMs, t x 1 vector                            @
@      charges         Fundcharges , t x 1 vector                             @

@      a               alpha prior                                            @
@      Aa              alpha variance prior                                   @
@      g               gamma prior                                            @
@      Ag              gamma variance prior                                   @
@                                                                             @
@      loops           number of Gibbs samples                                @
@      skip            number of samples to skip at start                     @
@      when            how often to update graph                              @

@-----------------------------------------------------------------------------@

new;
cls;
library pgraph;
graphset;
pqgwin manual;
format 8,5;
rndseed(1);

@ load data 1971-1 / 2005-4 @

load cred[140,1]=C:\dataufr\credit1971Qovgdp.txt;
cred=cred[14:140]';                          @ Fund credit  @
load cabal[140,1]=C:\dataufr\cabalovgdp.txt;
cabal=cabal[13:139]';
load corpchasp[128,1]=C:\dataufr\charges1974obs128.txt;
corpchasp=corpchasp[1:127]';
load termst[140,1]=C:\dataufr\termstructure.txt;
termst=termst[13:139]';
load textdebt[140,1]=C:\dataufr\totextdebtovgdp.txt;
textdebt=textdebt[13:139]';
load STextdebt[140,1]=C:\dataufr\totSTextdebtovres.txt;
stextdebt=STextdebt[13:139]';
load EMP[140,1]=C:\dataufr\emp1971q.txt;
emp=emp[13:139]';

@ Settings and preliminaries @

loops=10000;
skip=1000;
when=100;
t=cols(corpchasp);
t;

xaxis=seqa(1,1,t)';
as=zeros(7,loops-skip);
gs=zeros(3,loops-skip);
qs=zeros(t,loops-skip);
ps=zeros(t,loops-skip);
syav=0;

@ Uninformative prior for state @

n=rndu(1,t);
s=zeros(1,t);

i=1;
do until i>t;
	IF n[i]<0.5;
		s[i]=0;
    ELSE;
        s[i]=1;
    ENDIF;
i=i+1;
endo;

@ Uninformative prior for alpha @

a=(0|1|1|1|1|1|1);
Aa=diagrv(zeros(7,7),100*ones(7,1));

@ Prior for gamma @

g=zeros(3,1);
Ag=diagrv(zeros(3,3),100*ones(3,1));


@----------------------------------------------------------------------------@
@ Start Gibbs sampling loop                                                  @
@----------------------------------------------------------------------------@

k=1;
do until k>loops;

@---------------------------------------------------------------------------@
@ STEP 1: draw switching alpha parameters (F&G 4.2.1)                       @
@---------------------------------------------------------------------------@


@ Define endogenous and exogenous variables  @

s1=s[1:t];
d1=ones(1,t);
x1=(d1|s1|cabal|corpchasp|termst|textdebt|stextdebt);
yt=cred[1:t];
y1=yt[1:t];

@ Estimate and define residuals @

a=invpd(x1*x1')*(x1*y1');
Aa=invpd(x1*x1');
e=y1-a'x1;
Vy0=t;
Sy0=Vy0^-1*e*e';

@ Make a draw for the residual variance from inverted gamma distribution @

Syd=((Vy0/2)*Sy0.*(rndgam(1,1,Vy0/2))^-1);

@ Draw alpha from truncated normal distribution @

ad=zeros(2,1);
do until ad[2]>0;
       ad=a[1:2]+chol(Syd.*.Aa[2,2])'*rndn(2,1);
ENDO;
a[1:2]=ad;


@---------------------------------------------------------------------------@
@ STEP 2: draw state variable (st) recursively (F&G 4.2.2)                  @
@---------------------------------------------------------------------------@

q=zeros(1,t);
p=zeros(1,t);

@ Calculate smoothed probabilities q=prob(s[i]=0) and p=prob(s[i]=1) ... @

i=t;
DO UNTIL i<2;

       @ ... conditional on s[i-1] ... @

       latent=-g[1]-g[2]*emp[i]-g[3]*s[i-1];
       q[i]=cdfn(latent);
       p[i]=1-cdfn(latent);

       @ ... conditional on s[i+1] ... @

       IF i<t;
              latent=-g[1]-g[2]*emp[i];
              IF s[i+1]==0;
                     q[i]=q[i]*cdfn(latent);
                     p[i]=p[i]*cdfn(latent-g[3]);
              ELSE;
                     q[i]=q[i]*(1-cdfn(latent));
                     p[i]=p[i]*(1-cdfn(latent-g[3]));
              ENDIF;
       ENDIF;

       @ ... conditional on y[i] and s[j] for j<>i ... @

       IF i>t;

       ENDIF;

       s1=s;
       s2=s;
       s1[i]=0;
       s2[i]=1;
              z1=cred[i]-a'(1|s1[i]|cabal[i]|corpchasp[i]|termst[i]|textdebt[i]|stextdebt[i]);
              z2=cred[i]-a'(1|s2[i]|cabal[i]|corpchasp[i]|termst[i]|textdebt[i]|stextdebt[i]);
              q[i]=q[i]*exp(-0.5*(z1)'*inv(Syd)*(z1));
              p[i]=p[i]*exp(-0.5*(z2)'*inv(Syd)*(z2));

       @ Final definition of smoothed probabilities @

       q[i]=q[i]/(q[i]+p[i]);
       p[i]=1-q[i];

       @ Draw state st from uniform distribution @

       r=rndu(1,1);
       IF r<q[i];
              s[i]=0;
       ELSE;
              s[i]=1;
       ENDIF;

i=i-1;
ENDO;

@ Store results @

IF k>skip;
       qs[.,k-skip]=q';
	   ps[.,k-skip]=p';
	   syav=syav+syd;
       as[.,k-skip]=a;
ENDIF;

@---------------------------------------------------------------------------@
@ STEP 3: draw latent variable (ststar) (F&G 4.2.3)                         @
@---------------------------------------------------------------------------@

sstar=zeros(1,t);

@ Draw sstar from truncated normal @

i=2;
DO UNTIL i>t;
       latent=g[1]+g[2]*emp[i]+g[3]*s[i-1];
       r=rndn(1,1);
       IF s[i]==0;
              DO UNTIL latent+r<0;
                     r=rndn(1,1);
              ENDO;
       ELSE;
              DO UNTIL latent+r>0;
                     r=rndn(1,1);
              ENDO;
       ENDIF;
sstar[i]=latent+r;
i=i+1;
ENDO;

@---------------------------------------------------------------------------@
@ STEP 4: calculate posterior gamma distribution (F&G 4.2.4)                @
@---------------------------------------------------------------------------@

@ Define exogenous variables @

W=(ones(1,t)|emp[1:t]|s[1:t]);

@ Estimate @

Ag1=invpd(invpd(Ag)+W*W');
g1=Ag1*(invpd(Ag)*g+W*sstar[1:t]');

@ Draw gamma from normal distribution @

g=g1+chol(Ag1)'*rndn(3,1);
Ag=Ag1;

@ Print intermediate results @

locate 1,1;

IF k>skip;
       gs[.,k-skip]=g;
       if fmod(k,when)==0;
              print "Pass no." k;
              print "------------------------------------------------";
              print "Intermediate results";
              print "------------------------------------------------";
              print "alpha: a0  a1  cabal_1  corpchasp_1  termst_1  textdebt_1 stextdebt_1 ";
              print  (meanc(as[.,1:k-skip]')~stdc(as[.,1:k-skip]'))';
              print "------------------------------------------------";
              print "g: g0  EMP_1  st-1";
              print  (meanc(gs[.,1:k-skip]')~stdc(gs[.,1:k-skip]'))';
              print "------------------------------------------------";
              print "Sy" syav/(k-skip);
              print "------------------------------------------------";

              begwind;
              _ptitlht=0.6;
              window(3,1,1);

              title("dates");
              xy(xaxis',s');
              nextwind;

              title("smooth probability of tranquil regime");
              xy(xaxis',meanc(qs'));
			  nextwind;

              title("smooth probability of crisis regime");
              xy(xaxis',meanc(ps'));
              endwind;

       ENDIF;
ENDIF;
k=k+1;
ENDO;

@ Print final results @

k=k-1;
cls;
locate 1,1;
print "Pass no." k;
print "--------------------------------------------------------------";
print "Final results";
print "Number of Gibbs samples" k-skip;
print "---------------------------------------------------------------";
print "alpha: a0  a1  cabal_1  corpchasp_1 termst_1  textdebt_1 stextdebt_1  ";
print (meanc(as[.,1:k-skip]')~stdc(as[.,1:k-skip]'))';
print "------------------------------------------------";
print "g: g0  EMP_1  st-1";
print (meanc(gs[.,1:k-skip]')~stdc(gs[.,1:k-skip]'))';
print "---------------------------------------------------------------";
print "Sy" syav/(k-skip);
print "---------------------------------------------------------------";

gsm=meanc(gs[.,1:k-skip]');
gsm1=gsm[1,.];
gsm2=gsm[2,.];
gsm3=gsm[3,.];
empm=meanc(emp');
latent0=gsm1+(gsm2*empm);
latent1=latent0+gsm3;
print "Prob (st=0 | st-1=0)" 1-cdfn(latent0);
print "---------------------------------------------------------------";
print "Prob (st=1 | st-1=0)" cdfn(latent0);
print "---------------------------------------------------------------";
print "Prob (st=1 | st-1=1)" cdfn(latent1);
print "---------------------------------------------------------------";
print "Prob (st=0 | st-1=1)" 1-cdfn(latent1);
print "---------------------------------------------------------------";

begwind;
_ptitlht=0.6;
_pnum=1;
_plegctl=0;
_plegstr=0;
_pcolor=2;
_pnumht=0.25;

ytics(0,1,0.5,0);

_pbarwid=1;
_pbartyp[1,1]=6;


window(4,1,1);

scale(1|139,3|10000);
title("Use of Fund Credit");
xy(xaxis', cred');
nextwind;

scale(1|139,0|1);
title("dates");
xy(xaxis',s');
nextwind;

title("smooth probability of Fund low-lending regime");
xy(xaxis',meanc(qs[.,.]'));
nextwind;

title("smooth probability of Fund high-lending regime");
xy(xaxis',meanc(ps[.,.]'));
endwind;


@ y=a'(d1|s); @
@ window(1,1,1); @
@ scale(1|139,3|10000); @
@ title("Use of Fund Credit: fitted vs. actual data"); @
@ xy(xaxis',cred'~y'); @
@ endwind; @

@***************************************************************************@


